#
# Copyright (C) 2008 Jacob Peddicord <jacob@xeiso.com>
#
# This file is part of Xeiso.
#
# Xeiso is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Xeiso is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Xeiso.  If not, see <http://www.gnu.org/licenses/>.

import sys
import os
import time
import subprocess

import pyglet.app
from pyglet.window import Window, key
if __debug__:
    from pyglet.clock import ClockDisplay

import slider
import package
from gui import Throbber
from gui.arrows import Arrows

XPK_VERSION = 2

def setup():
    """
    Set up Xeiso for running. Opens a window, sets GUI elements, and loads
    the menu interface.
    """
    
    import menus.main
    import menus.xpkmanage
    import menus.games
    
    global win, arrows, throbber, sled, options
    
    # open the window
    if __debug__:
        # development mode, run in a window
        win = Window(width=800, height=600, caption="Xeiso")
    else:
        # production mode, run as a kiosk
        win = Window(fullscreen=True)
        # lock the mouse and keyboard
        win.set_exclusive_mouse(True)
        win.set_exclusive_keyboard(True)

    # display GUI context arrows
    arrows = Arrows(win)

    # create a throbber
    throbber = Throbber()
    throbber.active = False
    
    if options.xpk > 1:
        # if installing a package, load the package interface
        sled = slider.Slider(menus.xpkmanage.MenuPackageInstall())
    elif options.game:
        # if a game was specified, jump to the games menu
        sled = slider.Slider(menus.games.MenuGames())
    else:
        # otherwise load the welcome screen
        sled = slider.Slider(menus.main.MenuWelcome())


    if __debug__:
        # show the framerate in development mode
        fps_display = ClockDisplay()


    ### Events
    @win.event
    def on_draw():
        """
        Draw objects. Logic should go in update() below.
        """
        
        win.clear()
        
        sled.draw()
        arrows.draw()
        throbber.draw()
        
        if __debug__:
            # framerate
            fps_display.draw()

    @win.event
    def on_key_press(symbol, modifiers):
        # production mode? disable ESC
        if not __debug__ and symbol == key.ESCAPE:
            return True
            
        sled.menu.key_down(symbol)

    @win.event
    def on_key_release(symbol, modifiers):
        sled.menu.key_up(symbol)

    @win.event
    def on_text_motion(motion):
        sled.menu.key_motion(motion)


### Frame update
def update(dt):
    """
    Handle logic, animations, etc.
    """
    
    sled.update(dt)
    arrows.update(dt)
    throbber.update(dt)
    

def restart(args=[]):
    """
    Restart Xeiso if running in normal mode, optionally with a list of
    arguments.
    """
    
    global win, options
    
    win.clear()
    win.flip()
    
    # unlock the keyboard and mouse
    win.set_exclusive_mouse(False)
    win.set_exclusive_keyboard(False)
    
    if options.restart:
        # relaunch self, not anything else
        run = [os.path.abspath(sys.argv[0])]
        
        # keep debug mode if set
        if __debug__:
            run.insert(0, 'python')
        
        if args:
            run.extend(args)
        
        subprocess.Popen(run)
        
    pyglet.app.exit()
