#
# Copyright (C) 2008 Jacob Peddicord <jacob@xeiso.com>
#
# This file is part of Xeiso.
#
# Xeiso is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Xeiso is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Xeiso.  If not, see <http://www.gnu.org/licenses/>.

import pyglet.resource
import pyglet.graphics
from pyglet.sprite import Sprite
from pyglet.text import Label
from pyglet.gl import *

import xeiso


class TextLabel(Label):
    """
    Simplify pyglet's Label for basic use. Also adds rotation.
    """
    
    def __init__(self, text, x=None, y=None, font_size=36,
            color=(255,255,255,255), anchor_x="center", anchor_y="center",
            multiline=False, width=None, rotation=0):
        """
        Set some sane defaults, such as screen centered text.
        """
        
        if x == None:
            x = xeiso.win.width/2
        if y == None:
            y = xeiso.win.height/2
        
        if multiline:
            if width == None:
                width = xeiso.win.width - 200
        
        self.rotation = rotation
        
        Label.__init__(self, text, x=x, y=y, font_size=font_size,
                font_name="Sans", anchor_x=anchor_x, anchor_y=anchor_y,
                color=color, multiline=multiline, width=width)
    
    def draw(self):
        """ Wrap the draw function to enable rotated text. """
        
        # save the original x and y to restore later
        ox, oy = self.x, self.y
        
        glPushMatrix()
        
        # move to the text x and y
        glTranslatef(ox, oy, 0)
        
        # set the x and y to zero since the draw is now translated there
        self.x, self.y = 0, 0
        
        # rotate
        glRotatef(self.rotation, 0, 0, 1)
        
        # and finally draw
        super(TextLabel, self).draw()
        
        # then restore the original x and y values and the drawing matrix
        self.x, self.y = ox, oy
        
        glPopMatrix()


class Throbber:
    """
    A simple notification that Xeiso is doing something.
    """
    
    def __init__(self):
        self.image = pyglet.resource.image("throbber.png")
        self.image.anchor_x = self.image.width/2
        self.image.anchor_y = self.image.height/2
        
        self.sprite = Sprite(self.image)
        self.sprite.x = xeiso.win.width - 100
        self.sprite.y = 100
        self.sprite.scale = 2
        
        self.active = False
        self.anim_frame = self.anim_frames = 10  # spin every 20 frames
    
    def draw(self):
        if not self.active: return
        
        self.sprite.draw()
    
    def update(self, dt):
        if not self.active: return
        
        self.anim_frame -= 1
        if self.anim_frame == 0:
            self.anim_frame = self.anim_frames
            self._rotate()
    
    def _rotate(self):
        self.sprite.rotation += 90
        if self.sprite.rotation == 360:
            self.sprite.rotation = 0


