#
# Copyright (C) 2008 Jacob Peddicord <jacob@xeiso.com>
#
# This file is part of Xeiso.
#
# Xeiso is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Xeiso is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Xeiso.  If not, see <http://www.gnu.org/licenses/>.

import pyglet.resource
import pyglet.graphics
from pyglet.sprite import Sprite
from pyglet.gl import *

from xeiso.gui import TextLabel


class Arrows:
    
    def __init__(self, winobj):
        """
        Adds an arrow to the main interface.
        Side can be "left", "right", "up", or "down".
        """
        
        # save window width/height
        self.winwidth = winobj.width
        self.winheight = winobj.height
                
        # load sprites
        img = pyglet.resource.image("arrow.png")
        img.anchor_x = img.width/2
        img.anchor_y = img.height/2
        self.batch = pyglet.graphics.Batch()
        
        # amount of space between an arrow and screen edge
        border = 50
        
        self.sprite = {}
        self.sprite['right'] = Sprite(img,
                x=self.winwidth-border,
                y=self.winheight/2,
                batch=self.batch)
        self.sprite['right'].visible = False
        self.sprite['left'] = Sprite(img,
                x=border,
                y=self.winheight/2,
                batch=self.batch)
        self.sprite['left'].rotation = 180
        self.sprite['left'].visible = False
        self.sprite['up'] = Sprite(img,
                x=self.winwidth/2,
                y=self.winheight-border,
                batch=self.batch)
        self.sprite['up'].rotation = -90
        self.sprite['up'].visible = False
        self.sprite['down'] = Sprite(img,
                x=self.winwidth/2,
                y=border,
                batch=self.batch)
        self.sprite['down'].rotation = 90
        self.sprite['down'].visible = False
        
        # set the reference coords
        self.default = {}
        self.default['right'] = self.sprite['right'].x
        self.default['left'] = self.sprite['left'].x
        self.default['up'] = self.sprite['up'].y
        self.default['down'] = self.sprite['down'].y
        
        ## animation
        # idle movement
        self.idle_direction = 1   # 1=moving out, -1=moving in
        self.idle_pct = 0.0       # percent the animation has completed
        # fading
        self.fade_direction = {}  # 1=in 0=none -1=out
        self.fade_direction['right'] = 0
        self.fade_direction['left'] = 0
        self.fade_direction['up'] = 0
        self.fade_direction['down'] = 0
        # click effect
        self.click_direction = 0  # 1=growing 0=none -1=shrinking
        self.click_pct = 0.0
        self.click_arrow = None   # which arrow
        
        # text
        self.text = {}
        self.text['right'] = None
        self.text['left'] = None
        self.text['up'] = None
        self.text['down'] = None
        
    def draw(self):
        """
        Draw the arrow on a screen border if active.
        """
        
        glLoadIdentity()
        self.batch.draw()
                
        for single in self.text:
            if self.text[single] != None:
                self.text[single].draw()

    def update(self, dt):
        """
        Handle animation
        """
        
        ## Idle animation
        # acceleration
        if self.idle_direction == 1:
            if self.idle_pct >= 1:
                self.idle_pct = 1.0
                self.idle_direction = -1
            else:
                self.idle_pct += 0.01 + (1.0-self.idle_pct)/7
        elif self.idle_direction == -1:
            if self.idle_pct <= 0:
                self.idle_pct = 0.0
                self.idle_direction = 1
            else:
                self.idle_pct -= 0.01 + (1.0-self.idle_pct)/7
        
        # offset pixels by percent
        idle_offset = 3 * self.idle_pct
        
        # sprite coords
        self.sprite['right'].x = self.default['right'] + idle_offset
        self.sprite['left'].x = self.default['left'] - idle_offset
        self.sprite['up'].y = self.default['up'] + idle_offset
        self.sprite['down'].y = self.default['down'] - idle_offset
        
        ## Fade in/out animation
        for d in self.fade_direction:
            # out
            if self.fade_direction[d] == -1:
                self.sprite[d].opacity -= 25
                if self.sprite[d].opacity <= 0:
                    self.sprite[d].visible = False
                    self.sprite[d].opacity = 0
                    self.fade_direction[d] = 0
            # in
            elif self.fade_direction[d] == 1:
                self.sprite[d].opacity += 25
                if self.sprite[d].opacity >= 255:
                    self.sprite[d].opacity = 255
                    self.fade_direction[d] = 0
        
        ## Click animation
        # acceleration
        if self.click_direction == 1:
            if self.click_pct >= 1:
                self.click_pct = 1.0
                self.click_direction = -1   # grown enough? shrink back down.
            else:
                self.click_pct += 0.3
        elif self.click_direction == -1:
            if self.click_pct <= 0:
                self.click_pct = 0.0
                self.click_direction = 0    # shrank back down? stop moving.
            else:
                self.click_pct -= 0.3
        
        if self.click_direction != 0:
            # scale
            click_scale = 1 + self.click_pct * 0.2
            
            # set the arrow scale
            self.sprite[self.click_arrow].scale = click_scale
        
    def change(self, right=None, left=None, up=None, down=None):
        """
        Set multiple arrow states at once
        """
        
        def anim_trigger():
            """ Check for a need to trigger animation """
            if ds[d].__class__ is bool:
                check = ds[d]
            else:
                check = True
                
            if self.sprite[d].visible != check:   # are we changing?
                if self.sprite[d].visible:        # was it visible before?
                    self.fade_direction[d] = -1   # then fade it out
                else:
                    self.fade_direction[d] = 1    # otherwise fade in
                    self.sprite[d].visible = True # and make it visible
        
        ds = { 'right': right, 'left': left, 'up': up, 'down': down }
        
        # text
        for d in ds:
            if ds[d] != None:
                if ds[d].__class__ is bool:
                    anim_trigger()
                    
                    self.text[d] = None
                else:
                    anim_trigger()
                    
                    if d == 'right':
                        x, y = self.winwidth-10, self.winheight/2+15
                        ax, ay = 'right', 'center'
                        rotation = -52
                    elif d == 'left':
                        x, y = 10, self.winheight/2-15
                        ax, ay = 'left', 'center'
                        rotation = -48
                    elif d == 'up':
                        x, y = self.winwidth/2-10, self.winheight-10
                        ax, ay = 'right', 'center'
                        rotation = 38
                    elif d == 'down':
                        x, y = self.winwidth/2+10, 10
                        ax, ay = 'left', 'center'
                        rotation = 38
                    
                    self.text[d] = TextLabel(ds[d], font_size=14,
                            color=(255,255,255,140), rotation=rotation,
                            x=x, y=y, anchor_x=ax, anchor_y=ay)
    
    def click(self, arrow):
        """ Trigger a click animation """
        self.click_arrow = arrow
        self.click_direction = 1


