#
# Copyright (C) 2008 Jacob Peddicord <jacob@xeiso.com>
#
# This file is part of Xeiso.
#
# Xeiso is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Xeiso is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Xeiso.  If not, see <http://www.gnu.org/licenses/>.

import pyglet.graphics
from pyglet.gl import *

import xeiso
from xeiso.gui import TextLabel


class Dialog:
    """
    A simple dialog.
    """
    
    def __init__(self, text, width=-50, height=-50, multiline=False):
        """
        Calculate a dialog size. If width or height is negative, assume pixels
        from screen border.
        
        dhw - "dialog half-width"
        dhh - "dialog half-height"
        whw - "window half-width"
        whh - "window half-height"
        """
        
        self.whw = xeiso.win.width / 2
        self.whh = xeiso.win.height / 2
        
        # if negative, calculate width using distance from screen edge
        if width > 0:
            self.dhw = -width/2
        else:
            self.dhw = xeiso.win.width/2 + width
        
        # same as above
        if height > 0:
            self.dhh = -height/2
        else:
            self.dhh = xeiso.win.height/2 + height
        
        # multiline dialog? restrict the text flow
        if multiline:
            txtwidth = self.dhw * 0.75
        else:
            txtwidth = None
        self.text = TextLabel(text, font_size=24, x=0, y=0,
                multiline=multiline, width=txtwidth)
        
        # animation
        self.anim_direction = 1   # 1=appearing, 0=visible, -1=disappearing
        self.scale = 0.1
        self.gone = False
        
    def update(self, dt):
        """ Control animation """
        
        if self.anim_direction == 1:
            self.scale += 0.15
            if self.scale >= 1:
                self.scale = 1
                self.anim_direction = 0
        elif self.anim_direction == -1:
            self.scale -= 0.15
            if self.scale <= 0:
                self.scale = 1   # set to 1 so OpenGL doesn't error on 0
                self.anim_direction = 0
                self.gone = True # schedule for deletion
    
    def draw(self):
        """ Draw the dialog with OpenGL. """
        
        glPushMatrix()
        glTranslatef(self.whw, self.whh, 0.0)
        glScalef(self.scale, self.scale, 1.0)
        
        # enable alpha transparency
        glEnable(GL_BLEND)
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
        
        glColor4f(0.1, 0.1, 0.1, 0.95)
        pyglet.graphics.draw(4, GL_QUADS, ('v2i',
                (-self.dhw, -self.dhh,  # bottom left
                 self.dhw, -self.dhh,   # bottom right
                 self.dhw, self.dhh,    # top right
                 -self.dhw, self.dhh)   # top left
        ))
        
        # turn off alpha when done to not interfere with other drawings
        glDisable(GL_BLEND)
        
        self.text.draw()
        
        glPopMatrix()
    
    def dismiss(self, dt=0):
        self.anim_direction = -1
