#
# Copyright (C) 2008 Jacob Peddicord <jacob@xeiso.com>
#
# This file is part of Xeiso.
#
# Xeiso is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Xeiso is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Xeiso.  If not, see <http://www.gnu.org/licenses/>.

import sys
import os.path
import subprocess
import time

import pyglet.app
from pyglet.sprite import Sprite
from pyglet.window import key
from pyglet.clock import schedule_once

import xeiso
import xeiso.package
import xeiso.menus
import main
from xeiso.gui.dialog import Dialog
from xeiso.gui.arrows import Arrows
from xeiso.gui.icons import IconSet


current_game = None

class MenuGames(xeiso.menus.Menu):
    """ Display the game listing. """
    
    def __init__(self):
        """ Load a game listing """
        
        global current_game
        
        self.error_failed = None
        
        # never been run?
        if current_game == None:
            # if a game was specified, load that game
            if xeiso.options.game != None:
                i = 0
                for g in xeiso.package.games:
                    if g.id == xeiso.options.game:
                        current_game = i
                        break
                    i += 1
                
                # game not found
                if current_game == None:
                    print "Unable to locate", xeiso.options.game
                    current_game = 0
            else:
                current_game = 0
            
            # check for failed launch message
            if xeiso.options.failcode:
                game_name = xeiso.package.games[current_game].name
                self.error_failed = Dialog("An error occurred while running the game.")
                schedule_once(self.error_failed.dismiss, 3)
                
                xeiso.arrows.change(up=False, down=False, left=False, right=False)
        
        self.game = xeiso.package.games[current_game]
        
        # load the image
        self.sprite = Sprite(self.game.image)
        self.sprite.x = xeiso.win.width/2
        self.sprite.y = xeiso.win.height/2
        
        if self.game.players > 0:
            self.right = "Launch"
        else:
            self.right = False
        
        if not self.error_failed:
            xeiso.arrows.change(up=True, down=True, left="Options", right=self.right)
        
    def draw(self):
        self.sprite.draw()
        if self.error_failed:
            self.error_failed.draw()
    
    def update(self, dt):
        if self.error_failed:
            self.error_failed.update(dt)
            if self.error_failed.gone:
                self.error_failed = None
                xeiso.arrows.change(up=True, down=True, left="Options", right=self.right)
    
    def key_motion(self, motion):
        """
        Handle key to switch games
        """
        
        global current_game
        
        if self.error_failed:
            return
        
        # switch games with up & down
        if motion == key.MOTION_DOWN:
            if not xeiso.sled.animating:
                # if at the end, loop back
                if current_game == len(xeiso.package.games) - 1:
                    current_game = 0
                else:
                    current_game += 1
                xeiso.sled.change_menu(MenuGames(), "down")
        
        elif motion == key.MOTION_UP:
            if not xeiso.sled.animating:
                # if at the start, loop to the end
                if current_game == 0:
                    current_game = len(xeiso.package.games) - 1
                else:
                    current_game -= 1
                xeiso.sled.change_menu(MenuGames(), "up")
            
    
    def key_down(self, keysym):
        """
        Handle key presses for showing game details, leaving, etc
        """
        
        if self.error_failed:
            return
        
        if keysym == key.LEFT and not xeiso.sled.animating:
            xeiso.sled.change_menu(main.MenuOptions(), "left")
        
        elif keysym == key.RIGHT and not xeiso.sled.animating:
            if self.game.players > 0:
                xeiso.sled.change_menu(MenuGameSingle(), "right")


class MenuGameSingle(xeiso.menus.Menu):
    """ Show a single game banner """
    
    def __init__(self):
        """
        Load the game banner and initalize it.
        Also find out which icons to display.
        """
        
        xeiso.arrows.change(up=False, down=False, left="Back", right="Start")
        
        xeiso.package.games[current_game].show_banner()
        
        icons = []
        
        if int(xeiso.package.games[current_game].fullscreen):
            icons.append('fullscreen')
        else:
            icons.append('window')
        
        if int(xeiso.package.games[current_game].hwaccel):
            icons.append('hwaccel')
        
        if int(xeiso.package.games[current_game].joypad) > 0:
            icons.append('joypad')
        
        if int(xeiso.package.games[current_game].mouse) > 0:
            icons.append('mouse')
        
        if int(xeiso.package.games[current_game].network):
            icons.append('network')
        
        self.icons = IconSet(icons, x=15, y=15)
    
    def draw(self):
        """ Call the package draw function. """
        
        xeiso.package.games[current_game].draw()
        
        self.icons.draw()
        
    def update(self, dt):
        """ Call the package update, but only if slider isn't moving. """
        
        if not xeiso.sled.animating:
            xeiso.package.games[current_game].update(dt)
    
    def key_down(self, keysym):
        """ Control any key presses; this should not be handled by the game."""
        
        if keysym == key.LEFT and not xeiso.sled.animating:
            xeiso.package.games[current_game].hide_banner()
            xeiso.sled.change_menu(MenuGames(), "left")
        
        # activate menu to launch the game
        elif keysym == key.RIGHT and not xeiso.sled.animating:
            xeiso.package.games[current_game].hide_banner()
            xeiso.sled.change_menu(MenuGameLaunch(), "right")


class MenuGameLaunch(xeiso.menus.Menu):
    """ Launch the game's launch_game process. """
    
    def __init__(self):
        """ Hide the banner while the game is launched. """
        
        xeiso.arrows.change(up=False, down=False, left=False, right=False)
    
    def transition_done(self):
        """ Launch the game once the slider is done transitioning. """
        
        # launch the process
        launch = [
                os.path.join(os.path.dirname(os.path.abspath(sys.argv[0])),
                    'xeiso-launch'),
                os.path.join(
                    xeiso.package.games[current_game].path,
                    xeiso.package.games[current_game].id + '.run')
                ]
        
        if __debug__:
            launch.insert(0, 'python')
        
        subprocess.Popen(launch)
            
        pyglet.app.exit()
    
