#
# Copyright (C) 2008 Jacob Peddicord <jacob@xeiso.com>
#
# This file is part of Xeiso.
#
# Xeiso is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Xeiso is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Xeiso.  If not, see <http://www.gnu.org/licenses/>.

import sys
import os
import imp
from distutils import version

from pyglet.text.layout import TextLayout
from pyglet.text import decode_attributed
from pyglet.sprite import Sprite
from pyglet.window import key
from pyglet.clock import schedule_once
from pyglet.app import exit

import xeiso
import xeiso.menus
import xeiso.package
import xeiso.package.thread
import main
from xeiso.gui import TextLabel
from xeiso.gui.dialog import Dialog
from xeiso.gui.icons import IconSet
from xeiso.media import URLImageThread


current_package = None
packages = []

class MenuPackageListDownload(xeiso.menus.Menu):
    """
    Download the package list.
    """
    
    def __init__(self):
        """
        Set basic UI settings
        """
        
        global packages
        
        self.msg_downloading = TextLabel("Downloading information...",
                font_size=24)
        self.msg_wait = TextLabel("Please wait", font_size=16,
                y=self.msg_downloading.y-50)
        
        if not packages:
            xeiso.arrows.change(up=False, down=False, left=False, right=False)
            xeiso.throbber.active = True
        
        self.error = None
        self.thread_active = False
    
    def update(self, dt):
        global packages, current_package
        
        if self.thread_active:
            if not self.thread.isAlive():
                self.thread_active = False
                if self.thread.exc:
                    self.die(self.thread.exc)
                else:
                    self.load_list()
        
        if self.error:
            self.error.update(dt)
            if self.error.gone:
                self.error = None
                xeiso.sled.change_menu(main.MenuOptions(), "up")
    
    def draw(self):
        self.msg_downloading.draw()
        self.msg_wait.draw()
        
        # show an error dialog if there's a problem
        if self.error:
            self.error.draw()
    
    def transition_done(self):
        """
        Begin to download data once the menu is loaded
        """
        
        if packages:
            xeiso.sled.change_menu(MenuPackageList(), "right")
            return
        
        self.thread = xeiso.package.thread.PackageListThread()
        self.thread.start()
        self.thread_active = True
    
    def load_list(self):
        """
        Parse the package list, load it, and create text objects for it.
        """
        
        global packages, current_package
        
        current_package = 0
        packages = xeiso.package.parse_package_list(self.thread.config, xeiso.package.games)
        
        if not packages:
            self.die("No new games available.")
            return
        
        # document format
        format = '''{color (255,255,255,255)}{font_size 24}%s

{font_size 14}%s

{bold True}%s{bold False}'''
        
        i = 0
        for p in packages:
            if packages[i]['local'] == "1":
                local_required = "This game must be already installed on your system to run."
            else:
                local_required = "Game included."
            
            doc = decode_attributed(format % (
                    packages[i]['name'],
                    packages[i]['description'],
                    local_required,
                    ))
            
            packages[i]['layout'] = TextLayout(doc, multiline=True,
                    width=xeiso.win.width-200, height=xeiso.win.height-200)
            packages[i]['layout'].anchor_x = 'center'
            packages[i]['layout'].anchor_y = 'center'
            
            packages[i]['layout'].x = xeiso.win.width/2
            packages[i]['layout'].y = xeiso.win.height/2
            
            i += 1
        
        xeiso.sled.change_menu(MenuPackageList(), "right")
    
    def die(self, error):
        self.error = Dialog(error, width=500, height=300)
        xeiso.throbber.active = False
        
        # remove the dialog after a few seconds
        schedule_once(self.error.dismiss, 3)


class MenuPackageList(xeiso.menus.Menu):
    
    def __init__(self):
        global packages, current_package
        
        xeiso.arrows.change(up=True, down=True, left="Options", right="Download")
        
        # save the global current package number to this menu
        self.current = current_package
        
        # icons
        icons = []
        
        if int(packages[self.current]['fullscreen']):
            icons.append('fullscreen')
        else:
            icons.append('window')
        
        if int(packages[self.current]['hwaccel']):
            icons.append('hwaccel')
        
        if int(packages[self.current]['joypad']) > 0:
            icons.append('joypad')
        
        if int(packages[self.current]['mouse']) > 0:
            icons.append('mouse')
        
        if int(packages[self.current]['network']):
            icons.append('network')
        
        self.icons = IconSet(icons, x=15, y=15)
        
        # download/load the image
        if len(packages[self.current]['logo']) > 0:
            self.logo_thread = URLImageThread(packages[self.current]['logo'])
            if self.logo_thread.local_exists:
                xeiso.throbber.active = False
            else:
                xeiso.throbber.active = True
            
            self.logo_thread.start()
            self.logo_thread_run = True
        else:
            self.logo_thread_run = False
        
        self.sprite = None
        
    def update(self, dt):
        if self.logo_thread_run:
            if not self.logo_thread.isAlive():
                # anchor bottom-right
                self.logo_thread.image.anchor_x = self.logo_thread.image.width
                self.logo_thread.image.anchor_y = 0
                # sprite it
                self.sprite = Sprite(self.logo_thread.image,
                        x=xeiso.win.width-30, y=30)
                self.sprite.opacity = 100 # / 255
                # shut off the throbber
                xeiso.throbber.active = False
                self.logo_thread_run = False
    
    def draw(self):
        global packages
        
        if self.sprite:
            self.sprite.draw()
        
        packages[self.current]['layout'].draw()
        self.icons.draw()
    
    def key_motion(self, motion):
        """
        Handle key to switch games
        """
        
        global packages, current_package
        
        # switch games with up & down
        if motion == key.MOTION_DOWN:
            if not xeiso.sled.animating:
                # if at the end, loop back
                if self.current == len(packages) - 1:
                    current_package = 0
                else:
                    current_package += 1
                xeiso.sled.change_menu(MenuPackageList(), "down")
        
        elif motion == key.MOTION_UP:
            if not xeiso.sled.animating:
                # if at the start, loop to the end
                if self.current == 0:
                    current_package = len(packages) - 1
                else:
                    current_package -= 1
                xeiso.sled.change_menu(MenuPackageList(), "up")
    
    def key_down(self, keysym):
        if keysym == key.LEFT and not xeiso.sled.animating:
            xeiso.sled.change_menu(main.MenuOptions(), "left")
        if keysym == key.RIGHT and not xeiso.sled.animating:
            xeiso.sled.change_menu(MenuPackageDownload(), "right")


class MenuPackageDownload(xeiso.menus.Menu):
    """
    Download a single package, then restart Xeiso into installation mode.
    """
    
    def __init__(self):
        """
        Set UI activity indicators
        """
        
        global packages, current_package
        
        statusmsg = "Downloading %s..." % packages[current_package]['name']
        self.msg_downloading = TextLabel(statusmsg,
                font_size=24)
        self.msg_wait = TextLabel("This may take some time.",
                font_size=16, y=self.msg_downloading.y-50)
        
        xeiso.arrows.change(up=False, down=False, left=False, right=False)
        xeiso.throbber.active = True
        
        self.error = None
        self.success = None
        self.thread = None
    
    def update(self, dt):
        global packages, current_package
        
        if self.thread:
            if not self.thread.isAlive():
                if self.thread.exc:
                    self.die(self.thread.exc)
                else:
                    self.success = Dialog("Downloaded. Restarting.")
                    schedule_once(self.success.dismiss, 2)
                    xeiso.throbber.active = False
                    self.package_fname = self.thread.fname
                
                self.thread = None
        
        if self.error:
            self.error.update(dt)
            if self.error.gone:
                self.error = None
                xeiso.sled.change_menu(MenuPackageList(), "left")
        
        if self.success:
            self.success.update(dt)
            if self.success.gone:
                self.success = None
                xeiso.restart(['-i', self.package_fname])
    
    def draw(self):
        self.msg_downloading.draw()
        self.msg_wait.draw()
        
        # show an error dialog if there's a problem
        if self.error:
            self.error.draw()
        
        if self.success:
            self.success.draw()
    
    def transition_done(self):
        """
        Begin to download the package once the menu is loaded
        """
        
        global packages, current_package
        
        self.thread = xeiso.package.thread.PackageDownloadThread(
                packages[current_package]['url'])
        self.thread.start()
    
    def die(self, error):
        self.error = Dialog(error, width=500, height=300)
        xeiso.throbber.active = False
        
        # remove the dialog after a few seconds
        schedule_once(self.error.dismiss, 3)

class MenuPackageInstall(xeiso.menus.Menu):
    """
    Install a single package. Currently only activated from the command line.
    """
    
    def __init__(self):
        """ Preload the logo and exit text """
        
        self.thread = None
        self.show_install_msg = False
        
        self.dlg_error = None
        self.dlg_success = None
        
        pkg = os.path.basename(xeiso.options.xpk)
        self.dlg_install = Dialog("Install %s?" % pkg)
        self.msg_install = TextLabel("Installing...")
        
        xeiso.arrows.change(up=False, down=False, left="Exit", right="Install")
    
    def draw(self):
        if self.dlg_install:
            self.dlg_install.draw()
            
        if self.dlg_error:
            self.dlg_error.draw()
        
        if self.dlg_success:
            self.dlg_success.draw()
        
        if self.show_install_msg:
            self.msg_install.draw()
    
    def update(self, dt):
        if self.dlg_install:
            self.dlg_install.update(dt)
            if self.dlg_install.gone:
                self.dlg_install = None
                # begin install when the dialog is gone
                self.install()
        
        if self.thread:
            if not self.thread.isAlive():
                # done installing
                self.show_install_msg = False
                xeiso.throbber.active = False
                if self.thread.exc:
                    self.error(self.thread.exc[1][1])
                else:
                    self.dlg_success = Dialog("Game installed. Restarting.")
                    schedule_once(self.dlg_success.dismiss, 3)
                
                self.thread = None
        
        if self.dlg_error:
            self.dlg_error.update(dt)
            if self.dlg_error.gone:
                self.dlg_error = None
                exit()
        
        if self.dlg_success:
            self.dlg_success.update(dt)
            if self.dlg_success.gone:
                self.dlg_success = None
                xeiso.restart()
                    
    
    def key_down(self, keysym):
        if keysym == key.RIGHT and not xeiso.sled.animating and self.dlg_install:
            self.dlg_install.dismiss()
            xeiso.throbber.active = True
            self.show_install_msg = True
            xeiso.arrows.change(left=False, right=False)
        elif keysym == key.LEFT and not xeiso.sled.animating and self.dlg_install:
            exit()
    
    def install(self):
        """ Begin installing the package. """
        
        # crappy filetype check
        if os.path.splitext(xeiso.options.xpk)[1] != ".xpk":
            self.error("Not an XPK.")
            return
        
        self.thread = xeiso.package.thread.InstallThread(xeiso.options.xpk)
        self.thread.start()
    
    def error(self, text):
        self.show_install_msg = False
        xeiso.throbber.active = False
        self.dlg_error = Dialog("Error: %s" % text)
        schedule_once(self.dlg_error.dismiss, 3)

