#
# Copyright (C) 2008 Jacob Peddicord <jacob@xeiso.com>
#
# This file is part of Xeiso.
#
# Xeiso is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Xeiso is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Xeiso.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import dircache
import imp
import ConfigParser
from distutils import version

import pyglet.image
import pyglet.resource

import xeiso

if __debug__:
    import traceback


class Package:
    """
    An package is a file (.xpk) (or directory) containing:
        - __init__.py with a class derived from Package (this one)
        - the application binaries and data files
        - an image to display on the game selector
        - media files to accompany the game's launcher banner
    """
    
    def __init__(self):
        """ Deprecated. """
        
    def show_banner(self):
        pass
    
    def hide_banner(self):
        pass
    
    def update(self, dt):
        pass
    
    def draw(self):
        pass
        
    def load_logo(self, image):
        """ Load the channel selector image """
        
        try:
            self.image = self.image_centered(image)
        except:
            self.image = pyglet.resource.image("loaderror.png")
            # set players to 0 to disable launching
            self.players = 0
    
    def load_image(self, image):
        """ Convenience function to load an image on the package path """
        
        filename = os.path.join(self.path, image)
        img = pyglet.image.load(filename)
        
        return img
    
    def image_centered(self, image):
        """ Convenience function to load an image with centered anchors. """
        
        img = self.load_image(image)
        img.anchor_x = img.width/2
        img.anchor_y = img.height/2
        
        return img


### Packaging-related functions
def parse_package_list(pkgconfig, loaded_packages):
    """
    Run through SafeConfigParser pkglist and generate a list of packages.
    Remove any already installed and not upgradable.
    """
    
    # get a list of installed packages
    installed_packages = []
    for pkg in loaded_packages:
        installed_packages.append(pkg.id)
    
    pkglist = []
    
    for section in pkgconfig.sections():
        items = pkgconfig.items(section)
        
        pkg = {'id': section}
        for i in items:
            pkg[i[0]] = i[1]
        
        # version check
        if pkg['id'] in installed_packages:
            pkg_index = installed_packages.index(pkg['id'])
            
            local_version = version.LooseVersion(loaded_packages[pkg_index].version)
            new_version = version.LooseVersion(pkg['version'])

            if local_version >= new_version:
                #local version is up-to-date or newer
                continue
        
        pkglist.append(pkg)
    
    return pkglist


def load_games(dirs):
    """ 
    Load games from the xpk directory and returns them as a list of
    Package objects.
    """
    
    games = []
    names = []
    
    for d in dirs:
        if __debug__: print "=== Loading XPKs in:", d
        
        files = dircache.listdir(d)
        
        for f in files:
            if os.path.isdir(os.path.join(d, f)):
                if __debug__:
                    print "---", f
                
                # check to see if already loaded
                if f in names:
                    if __debug__: "* Already loaded, skipping"
                    continue
                
                # find the module
                try:
                    fp, pathname, desc = imp.find_module(f, [d])
                except:
                    if __debug__:
                        print "*", f, "is a binary package and cannot be loaded in debug mode."
                    continue
                
                # load the module
                try:
                    mod = imp.load_module("xpk."+f, fp, pathname, desc)
                except:
                    print "* Failed to load module", f
                    if __debug__:
                        traceback.print_exc()
                    continue
                finally:
                    if fp: fp.close()
                
                # parse the ini
                config = ConfigParser.SafeConfigParser()
                ini = os.path.join(d, f, f + '.ini')
                try:
                    config.read([ini])
                    section = config.sections()[0]
                except:
                    print "* Failed to load config for", f
                    if __debug__:
                        traceback.print_exc()
                    continue
                
                # check spec version
                spec_got = config.getint(section, 'spec')
                if spec_got != xeiso.XPK_VERSION:
                    print "*", f, "is not compatible with this version of Xeiso."
                    if __debug__:
                        print "  Required version", spec_req, "got version", spec_got
                    continue
                    
                # load the Package instance
                pkg = mod.Package()
                pkg.path = pathname
                pkg.name = section
                pkg.id = f
                
                # save package config to pkg
                for i in config.items(section):
                    pkg.__dict__[i[0]] = i[1]
                
                # load the game logo
                logo = os.path.join(pathname, pkg.logo)
                pkg.load_logo(logo)
                
                # add to the game listing
                games.append(pkg)
                names.append(f)
    
    return games
